@@ -709,3 +709,57 @@ class Convolvetests(TestCase):
         actual = mi.take(5, mi.convolve(signal, kernel))
         expected = [0, 1, 1, 1, 1]
         self.assertEqual(actual, expected)
+
+
+class BeforeAndAfterTests(TestCase):
+    def test_empty(self):
+        before, after = mi.before_and_after(bool, [])
+        self.assertEqual(list(before), [])
+        self.assertEqual(list(after), [])
+
+    def test_never_true(self):
+        before, after = mi.before_and_after(bool, [0, False, None, ''])
+        self.assertEqual(list(before), [])
+        self.assertEqual(list(after), [0, False, None, ''])
+
+    def test_never_false(self):
+        before, after = mi.before_and_after(bool, [1, True, Ellipsis, ' '])
+        self.assertEqual(list(before), [1, True, Ellipsis, ' '])
+        self.assertEqual(list(after), [])
+
+    def test_some_true(self):
+        before, after = mi.before_and_after(bool, [1, True, 0, False])
+        self.assertEqual(list(before), [1, True])
+        self.assertEqual(list(after), [0, False])
+
+
+class TriplewiseTests(TestCase):
+    def test_basic(self):
+        for iterable, expected in [
+            ([0], []),
+            ([0, 1], []),
+            ([0, 1, 2], [(0, 1, 2)]),
+            ([0, 1, 2, 3], [(0, 1, 2), (1, 2, 3)]),
+            ([0, 1, 2, 3, 4], [(0, 1, 2), (1, 2, 3), (2, 3, 4)]),
+        ]:
+            with self.subTest(expected=expected):
+                actual = list(mi.triplewise(iterable))
+                self.assertEqual(actual, expected)
+
+
+class SlidingWindowTests(TestCase):
+    def test_basic(self):
+        for iterable, n, expected in [
+            ([], 0, [()]),
+            ([], 1, []),
+            ([0], 1, [(0,)]),
+            ([0, 1], 1, [(0,), (1,)]),
+            ([0, 1, 2], 2, [(0, 1), (1, 2)]),
+            ([0, 1, 2], 3, [(0, 1, 2)]),
+            ([0, 1, 2], 4, []),
+            ([0, 1, 2, 3], 4, [(0, 1, 2, 3)]),
+            ([0, 1, 2, 3, 4], 4, [(0, 1, 2, 3), (1, 2, 3, 4)]),
+        ]:
+            with self.subTest(expected=expected):
+                actual = list(mi.sliding_window(iterable, n))
+                self.assertEqual(actual, expected)
